@@ -1,12 +1,10 @@
 
-from base64 import b64encode
-from urllib import urlencode
-
 import urllib2
 
 from exceptions import Exception
 
 from twitter.twitter_globals import POST_ACTIONS
+from twitter.auth import UserPassAuth, NoAuth
 
 def _py26OrGreater():
     import sys
@@ -26,22 +24,26 @@ class TwitterError(Exception):
 
 class TwitterCall(object):
     def __init__(
-        self, username, password, format, domain, uri="", agent=None):
-        self.username = username
-        self.password = password
+        self, auth, format, domain, uri="", agent=None,
+        encoded_args=None, secure=True):
+        self.auth = auth
         self.format = format
         self.domain = domain
         self.uri = uri
         self.agent = agent
+        self.encoded_args = encoded_args
+        self.secure = secure
+
     def __getattr__(self, k):
         try:
             return object.__getattr__(self, k)
         except AttributeError:
             return TwitterCall(
-                self.username, self.password, self.format, self.domain,
-                self.uri + "/" + k, self.agent)
+                self.auth, self.format, self.domain,
+                self.uri + "/" + k, self.agent, self.encoded_args, self.secure)
+
     def __call__(self, **kwargs):
-        uri = self.uri
+        uri = self.uri.strip("/")
         method = "GET"
         for action in POST_ACTIONS:
             if self.uri.endswith(action):
@@ -50,30 +52,37 @@ class TwitterCall(object):
                     kwargs["source"] = self.agent
                 break
 
-        id = kwargs.pop('id', None)
-        if id:
-            uri += "/%s" %(id)
+        secure_str = ''
+        if self.secure:
+            secure_str = 's'
+        dot = ""
+        if self.format != '':
+            dot = "."
+        uriBase = "http%s://%s/%s%s%s" %(
+                    secure_str, self.domain, uri, dot, self.format)
+
+        if (not self.encoded_args):
+            if kwargs.has_key('id'):
+                uri += "/%s" %(kwargs['id'])
+
+            self.encoded_args = self.auth.encode_params(uriBase, method, kwargs)
 
         argStr = ""
         argData = None
-        encoded_kwargs = urlencode(kwargs.items())
         if (method == "GET"):
-            if kwargs:
-                argStr = "?%s" %(encoded_kwargs)
+            if self.encoded_args:
+                argStr = "?%s" %(self.encoded_args)
         else:
-            argData = encoded_kwargs
+            argData = self.encoded_args
 
         headers = {}
         if (self.agent):
             headers["X-Twitter-Client"] = self.agent
-        if (self.username):
-            headers["Authorization"] = "Basic " + b64encode("%s:%s" %(
-                self.username, self.password))
+        if self.auth is not None:
+            headers.update(self.auth.generate_headers())
 
-        req = urllib2.Request(
-                "http://%s/%s.%s%s" %(self.domain, uri, self.format, argStr),
-                argData, headers
-            )
+        req = urllib2.Request(uriBase+argStr, argData, headers)
+        
         try:
             handle = urllib2.urlopen(req)
             if "json" == self.format:
@@ -86,7 +95,7 @@ class TwitterCall(object):
             else:
                 raise TwitterError(
                     "Twitter sent status %i for URL: %s.%s using parameters: (%s)\ndetails: %s" %(
-                        e.code, uri, self.format, encoded_kwargs, e.fp.read()))
+                        e.code, uri, self.format, self.encoded_args, e.fp.read()))
 
 class Twitter(TwitterCall):
     """
@@ -153,14 +162,25 @@ class Twitter(TwitterCall):
     """
     def __init__(
         self, email=None, password=None, format="json", domain="twitter.com",
-        agent=None):
+        agent=None, secure=True, auth=None):
         """
         Create a new twitter API connector using the specified
         credentials (email and password). Format specifies the output
         format ("json" (default) or "xml").
         """
-        if (format not in ("json", "xml")):
+        
+        if email is not None or password is not None:
+            if auth is not None:
+                raise ValueError, "can't specify 'email' or 'password' and 'auth' params"
+            auth = UserPassAuth(email, password)
+
+        if not auth:
+            auth = NoAuth()
+
+        if (format not in ("json", "xml", "")):
             raise TwitterError("Unknown data format '%s'" %(format))
-        TwitterCall.__init__(self, email, password, format, domain, "", agent)
+        TwitterCall.__init__(
+            self, auth, format, domain, "", agent, 
+            secure=secure)
 
 __all__ = ["Twitter", "TwitterError"]
